#include "plumpconfig.h"
#include "mediums.h"

int plump_mediumIsSupported(plumpmedium_t medium)
{
	int ret = PLUMP_ERROR_NOTSUPPORTEDMEDIUM;

	switch (medium)
	{
		case PLUMP_MEDIUM_ANY:
#if defined(MEDIUM_FAVOR_PTRACE) && defined(MEDIUM_HAS_PTRACE)
			ret = plump_mediumIsSupported(PLUMP_MEDIUM_PTRACE);
#elif defined(MEDIUM_FAVOR_PROCFS) && defined(MEDIUM_HAS_PROCFS)
			ret = plump_mediumIsSupported(PLUMP_MEDIUM_PROCFS);
#endif

#if defined(MEDIUM_HAS_PTRACE)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumIsSupported(PLUMP_MEDIUM_PTRACE);
#endif

#if defined(MEDIUM_HAS_PROCFS)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumIsSupported(PLUMP_MEDIUM_PROCFS);
#endif

#if defined(MEDIUM_HAS_DBGAPI)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumIsSupported(PLUMP_MEDIUM_DBGAPI);
#endif
			break;
		case PLUMP_MEDIUM_PTRACE:
#ifdef MEDIUM_HAS_PTRACE
			ret = ptrace_okay();
#endif
			break;
		case PLUMP_MEDIUM_PROCFS:
#ifdef MEDIUM_HAS_PROCFS
			ret = procfs_okay();
#endif
			break;
		case PLUMP_MEDIUM_DBGAPI:
#ifdef MEDIUM_HAS_DBGAPI
			ret = dbgapi_okay();
#endif
		default:
			break;
	}

	return ret;
}

int plump_mediumGetFunctionTable(plumpmedium_t medium, plumpfunc_t *functions)
{
	int ret = PLUMP_ERROR_NOTSUPPORTEDMEDIUM;

	switch (medium)
	{
		case PLUMP_MEDIUM_ANY:
#if defined(MEDIUM_FAVOR_PTRACE) && defined(MEDIUM_HAS_PTRACE)
			ret = plump_mediumGetFunctionTable(PLUMP_MEDIUM_PTRACE, functions);
#elif defined(MEDIUM_FAVOR_PROCFS) && defined(MEDIUM_HAS_PROCFS)
			ret = plump_mediumGetFunctionTable(PLUMP_MEDIUM_PROCFS, functions);
#endif

#if defined(MEDIUM_HAS_PTRACE)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumGetFunctionTable(PLUMP_MEDIUM_PTRACE, functions);
#endif

#if defined(MEDIUM_HAS_PROCFS)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumGetFunctionTable(PLUMP_MEDIUM_PROCFS, functions);
#endif

#if defined(MEDIUM_HAS_DBGAPI)
			if (ret == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
				ret = plump_mediumGetFunctionTable(PLUMP_MEDIUM_DBGAPI, functions);
#endif

			break;
		case PLUMP_MEDIUM_PTRACE:
#ifdef MEDIUM_HAS_PTRACE
			functions->getMedium             = ptrace_getMedium;
			functions->open                  = ptrace_open;
			functions->close                 = ptrace_close;
			functions->execContinue          = ptrace_execContinue;
			functions->execStop              = ptrace_execStop;
			functions->memoryWrite           = ptrace_memoryWrite;
			functions->memoryRead            = ptrace_memoryRead;
			functions->memoryIsValid         = ptrace_memoryIsValid;
			functions->memoryGetEndOfSegment = ptrace_memoryGetEndOfSegment;
			functions->registersSet          = ptrace_registersSet;
			functions->registersGet          = ptrace_registersGet;

			ret = PLUMP_SUCCESS;
#endif
			break;
		case PLUMP_MEDIUM_PROCFS:
#ifdef MEDIUM_HAS_PROCFS
			functions->getMedium             = procfs_getMedium;
			functions->open                  = procfs_open;
			functions->close                 = procfs_close;
			functions->execContinue          = procfs_execContinue;
			functions->execStop              = procfs_execStop;
			functions->memoryWrite           = procfs_memoryWrite;
			functions->memoryRead            = procfs_memoryRead;
			functions->memoryIsValid         = procfs_memoryIsValid;
			functions->memoryGetEndOfSegment = procfs_memoryGetEndOfSegment;
			functions->registersSet          = procfs_registersSet;
			functions->registersGet          = procfs_registersGet;
			
			ret = PLUMP_SUCCESS;
#endif
			break;
		case PLUMP_MEDIUM_DBGAPI:
#ifdef MEDIUM_HAS_DBGAPI
			functions->getMedium             = dbgapi_getMedium;
			functions->open                  = dbgapi_open;
			functions->close                 = dbgapi_close;
			functions->execContinue          = dbgapi_execContinue;
			functions->execStop              = dbgapi_execStop;
			functions->memoryWrite           = dbgapi_memoryWrite;
			functions->memoryRead            = dbgapi_memoryRead;
			functions->memoryIsValid         = dbgapi_memoryIsValid;
			functions->memoryGetEndOfSegment = dbgapi_memoryGetEndOfSegment;
			functions->registersSet          = dbgapi_registersSet;
			functions->registersGet          = dbgapi_registersGet;
			
			ret = PLUMP_SUCCESS;
#endif
			break;
		default:
			break;
	}

	return ret;
}
